/**
 *
 * This file is part of jReality. jReality is open source software, made
 * available under a BSD license:
 *
 * Copyright (c) 2003-2006, jReality Group: Charles Gunn, Tim Hoffmann, Markus
 * Schmies, Steffen Weissmann.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * - Neither the name of jReality nor the names of its contributors nor the
 *   names of their associated organizations may be used to endorse or promote
 *   products derived from this software without specific prior written
 *   permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */


package de.jreality.tutorial;

import java.awt.Color;
import java.io.IOException;

import de.jreality.examples.CatenoidHelicoid;
import de.jreality.math.MatrixBuilder;
import de.jreality.scene.Appearance;
import de.jreality.scene.IndexedFaceSet;
import de.jreality.scene.SceneGraphComponent;
import de.jreality.shader.CommonAttributes;
import de.jreality.shader.ImageData;
import de.jreality.shader.Texture2D;
import de.jreality.shader.TextureUtility;
import de.jreality.ui.viewerapp.ViewerApp;
import de.jreality.util.Input;

public class TextureExample {
  
  // java de.jreality.tutorial.TextureExample grid.jpeg
  
  public static void main(String[] args) throws IOException {
	  IndexedFaceSet ico = new CatenoidHelicoid(40);
	  SceneGraphComponent sgc = new SceneGraphComponent();
	  sgc.setGeometry(ico);
      sgc.setAppearance(new Appearance());
      sgc.getAppearance().setAttribute(CommonAttributes.VERTEX_DRAW, false);
      sgc.getAppearance().setAttribute(CommonAttributes.EDGE_DRAW, false);
      ImageData id = null;
      sgc.getAppearance().setAttribute(CommonAttributes.LINE_SHADER+"."+CommonAttributes.DIFFUSE_COLOR, Color.GREEN);
      double scale = 3;
      if (args.length > 0) {
    	  id =  ImageData.load(Input.getInput(args[0]));
          sgc.getAppearance().setAttribute(CommonAttributes.DIFFUSE_COLOR, Color.WHITE);
          scale = 1;
      }
      else  {
    	  SimpleTextureFactory stf = new SimpleTextureFactory();
    	  stf.update();
    	  id = stf.getImageData();
    	  scale = 10;
          sgc.getAppearance().setAttribute(CommonAttributes.DIFFUSE_COLOR, Color.YELLOW);
     }
      Texture2D tex = TextureUtility.createTexture(sgc.getAppearance(), CommonAttributes.POLYGON_SHADER,id);
      tex.setTextureMatrix(MatrixBuilder.euclidean().scale(scale).getMatrix());
      tex.setApplyMode(Texture2D.GL_MODULATE);
    	
    ViewerApp.display(sgc);
  }
}